import logging
import os
import subprocess
from pathlib import Path
from typing import List, Optional

import configparser

from snakemake.script import Snakemake

from omnibenchmark.benchmark.params import Params


def mock_execution(inputs: List[str], output: str, snakemake: Snakemake):
    print("Processed", inputs, "to", output, "using threads", snakemake.threads)
    print("  bench_iteration is", snakemake.bench_iteration)
    print("  resources are", snakemake.resources)
    print("  wildcards are", snakemake.wildcards)
    print("  rule is", snakemake.rule)
    print("  scriptdir is", snakemake.scriptdir)
    print("  params are", snakemake.params)


def execution(
    module_dir: Path,
    module_name: str,
    output_dir: Path,
    dataset: str,
    inputs_map: dict[str, str | List[str]],
    parameters: Params,
    keep_module_logs: bool,
    timeout: int,
) -> int:
    config_parser = _read_config(module_dir)
    if config_parser is None:
        logging.error(f"ERROR: Failed to load config.cfg for {module_name}.")
        return -1  # return exit_code -1

    executable = config_parser["DEFAULT"]["SCRIPT"]
    executable_path = module_dir / executable
    if not os.path.exists(executable_path):
        logging.error(f"ERROR: {module_name} executable does not exist.")
        return -1  # return exit_code -1

    # Constructing the command
    command = _create_command(executable_path)
    if command is None:
        return -1  # return exit_code -1

    # Add output directory and dataset name to command
    command.extend(["--output_dir", output_dir.as_posix(), "--name", dataset])

    # Adding input files with their respective keys
    if inputs_map:
        for k, v in inputs_map.items():
            if isinstance(v, str):
                command.extend([f"--{k}", Path(v).as_posix()])
            elif isinstance(v, list) and all(isinstance(item, str) for item in v):
                command.extend([f"--{k}"] + [Path(item).as_posix() for item in v])

    # Adding extra parameters
    if parameters:
        command.extend(parameters.to_cli_args())

    # Prepare stdout and stderr files in the output directory
    stdout_file = output_dir / "stdout.log"
    stderr_file = output_dir / "stderr.log"

    logging.info(f"Seting timeout to {timeout} seconds")

    try:
        with open(stdout_file, "w") as stdout_f, open(stderr_file, "w") as stderr_f:
            result = subprocess.run(
                command,
                check=True,
                stdout=stdout_f,
                stderr=stderr_f,
                text=True,
                timeout=timeout,
            )
            exit_code = result.returncode
            logging.info(f"{executable} ran successfully with return code {exit_code}.")

    except subprocess.CalledProcessError as e:
        exit_code = e.returncode
        logging.error(
            f"ERROR: Executing {executable} failed with exit code {e.returncode}. See {stderr_file} for details."
        )

    except subprocess.TimeoutExpired as e:
        logging.error(
            f"ERROR: Executing {executable} failed after timing out in {timeout}s"
            f"See {stderr_file} for details."
        )
        raise e

    finally:
        # Cleanup empty log files / always cleanup stdout_file if keep_module_logs is False
        if stdout_file.exists() and (
            os.path.getsize(stdout_file) == 0 or not keep_module_logs
        ):
            stdout_file.unlink()
        if stderr_file.exists() and os.path.getsize(stderr_file) == 0:
            stderr_file.unlink()

    if exit_code is None:
        logging.error(f"ERROR: Failed to execute {executable}: Unknown error")
        exit_code = -1

    return exit_code


def _read_config(module_dir: Path) -> Optional[configparser.ConfigParser]:
    config_path = module_dir / "config.cfg"
    if not os.path.exists(config_path):
        return None

    parser = configparser.ConfigParser()

    # Read the configuration file
    parser.read(config_path)

    return parser


def _create_command(executable_path: Path) -> Optional[list[str]]:
    _, extension = os.path.splitext(executable_path)

    if extension == ".py":
        # Execute Python script
        return ["python3", executable_path.as_posix()]
    elif extension == ".R":
        # Execute R script
        return ["Rscript", executable_path.as_posix()]
    elif extension == ".sh":
        # Execute shell script
        return ["sh", executable_path.as_posix()]
    else:
        logging.error(
            f"ERROR: Unsupported script extension: {extension}. Only Python/R/Shell scripts are supported."
        )
        return None



"""General utils functions"""

import os

from linkml_runtime.loaders import yaml_loader
import subprocess
from pathlib import Path
from typing import List, Union, Any
import yaml
import platform

from omni_schema.datamodel.omni_schema import IOFile


def try_avail_envmodule(module_name: str) -> bool:
    env = {}
    env.update(os.environ)

    command = f"""
    . "$LMOD_PKG"/init/profile ;
    module purge ;
    module avail {module_name}"""
    result = subprocess.run(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        shell=True,
        text=True,
        env=env,
    )

    return "No module(s) or extension(s) found!" not in result.stderr


def as_list(input: Union[List, Any]):
    return input if isinstance(input, List) else [input]


def parse_instance(path: Path, target_class) -> Any:
    """Load a model of target_class from a file."""

    # Due to a yaml.CLoader issue for Yaml files on Windows, https://github.com/yaml/pyyaml/issues/293
    # We will have different logic for loading the model based on the OS.
    # Basically the Windows user will have a slower loading time, because the yaml.Loader does not have the issue
    if platform.system() == "Windows":
        with path.open("r") as file:
            benchmark_yaml = yaml.load(file, yaml.SafeLoader)
            benchmark = yaml_loader.load(benchmark_yaml, target_class)
            return benchmark
    else:
        benchmark = yaml_loader.load(str(path), target_class)
        return benchmark


def merge_dict_list(list_of_dicts):
    """Merge a list of dictionaries into a single dictionary."""
    merged_dict = {
        key: value for d in list_of_dicts if d is not None for key, value in d.items()
    }

    return merged_dict


def format_mc_output(output: IOFile, out_dir: Path, collector_id: str):
    if output.path:
        o = output.path.replace("{input}", str(out_dir))
        o = o.replace("{name}", collector_id)
        return o
    else:
        return str(out_dir / output.id)



import os
from pathlib import Path
from filelock import FileLock

from .params import Params

UNSAFE_CHARS = {
    "/": "_",
    "\\": "_",
    ":": "_",
    "*": "_",
    "?": "_",
    '"': "_",
    "<": "_",
    ">": "_",
    "|": "_",
    " ": "_",
}


class SymlinkManager:
    """
    Manages a two-level storage system for parameters with human-readable access.

    This class implements a storage pattern where parameters are stored in a content-addressable
    storage (using their hash as identifier) while maintaining human-readable symbolic links
    for easier browsing and access.

    The storage structure is:
        {base_dir}/               # Human-readable symlinks
            a_1-b_2-c_3/          # -> {.hash123}
            x_1-y_2/              # -> {.hash456}
            {.hash123}/           # Actual storage
                parameters.json
            {.hash456}/
                parameters.json

    Args:
        base_dir (str): Base directory for storage

    Example:
        >>> manager = SymlinkManager("./data")
        >>> info = manager.store(params)
        >>> print(f"Stored in {info['human']}, hash: {info['folder']}")
        >>> retrieved_params = manager.get_params("param_a1_b2_c3")
    """

    def __init__(self, base_dir: str | Path = "."):
        self.base_dir: Path = Path(base_dir)
        self.lock = FileLock(self.base_dir / "storage.lock")

    def _sanitize_name(self, name):
        """Basic character sanitization for filesystem safety."""
        for unsafe, safe in UNSAFE_CHARS.items():
            name = name.replace(unsafe, safe)
        return name

    def _make_human_name(self, params, max_len=255):
        """
        Create human-readable name, falling back appending short hash if too long.

        Args:
            params: Params object to create name from
            max_len (int): Maximum length of the folder name.

        Returns:
            str: Human-readable name within filesystem constraints
        """
        parts = [f"{k}-{v}" for k, v in params.items()]
        name = "_".join(parts)
        name = self._sanitize_name(name)

        # If too long, append a short 8 characters hash
        if len(name) > max_len:
            short_hash = params.hash()[:8]
            name = name[: max_len - 9] + "_" + short_hash
        return name

    def _write_params(self, params, hash_path, human_path):
        params_hash = params.hash()
        params_file = hash_path / "parameters.json"
        with open(params_file, "w") as f:
            f.write(params.serialize())

        params_dict_tsv = hash_path.parent / "parameters_dict.tsv"
        if os.path.exists(params_dict_tsv):
            with open(params_dict_tsv, "r") as file:
                existing_lines = file.readlines()

            # If the params_hash exists, skip writing
            if any(params_hash in line for line in existing_lines):
                return
        else:
            with open(params_dict_tsv, "w") as file:
                file.write("id\tbase_path\talias_path\tparameters\n")

        # If not found, write the new entry to parameters_dict.tsv
        with open(params_dict_tsv, "a") as file:
            file.write(
                f"{params_hash}\t{hash_path}\t{human_path}\t{params.serialize()}\n"
            )

    def store(self, params):
        """
        Store parameters and create human-readable symlink.

        Args:
            params: Params object to store

        Returns:
            dict: Information about stored params including:
                - folder: Hash-based folder name
                - params: Original params object
                - human: Human-readable name
        """
        if params:
            with self.lock:
                hash_id = params.hash()
                hash_folder = f".{hash_id}"
                human_name = self._make_human_name(params)

                # Create hash directory and store params
                hash_path = self.base_dir / hash_folder
                hash_path.mkdir(exist_ok=True)

                # Create/update symlink
                symlink_path = self.base_dir / human_name
                if symlink_path.exists() or symlink_path.is_symlink():
                    symlink_path.unlink()

                # Create relative path from visible_dir to hidden_dir
                real_target = self.base_dir / hash_folder
                relative_path = os.path.relpath(real_target, symlink_path.parent)
                symlink_path.symlink_to(relative_path)

                self._write_params(params, hash_path, symlink_path)

                return {"folder": hash_folder, "params": params, "human": human_name}

    def get_params(self, human_name):
        """
        Retrieve parameters from human-readable name.
        """
        symlink_path = self.base_dir / human_name
        if not symlink_path.exists():
            raise FileNotFoundError(f"No parameters found for name: {human_name}")

        # Follow symlink to get the actual parameters.json file
        target = symlink_path.readlink()
        params_file = symlink_path.parent / target / "parameters.json"

        if not params_file.exists():
            raise ValueError(
                f"Parameters file missing for: {human_name} (looking in {params_file})"
            )

        with open(params_file) as f:
            return Params.deserialize(f.read())



from pathlib import Path
from typing import Union, Optional, Dict, List

from omni_schema.datamodel import omni_schema

from omnibenchmark.benchmark import params
from omnibenchmark.utils import merge_dict_list, parse_instance


class LinkMLConverter:
    def __init__(self, benchmark_file: Path):
        """
        The constructor tries to parse the benchmark file and initialize the model.
        It also validates the model against the schema, and raises a ValidationError if
        """
        self.benchmark_file = benchmark_file
        self.model: omni_schema.Benchmark = parse_instance(
            benchmark_file, omni_schema.Benchmark
        )

    def get_name(self) -> str:
        """Get name of the benchmark"""

        return self.model.name if self.model.name else self.model.id

    def get_version(self) -> str:
        """Get version of the benchmark"""

        return self.model.version

    def get_author(self) -> str:
        """Get author of the benchmark"""

        return self.model.benchmarker

    def get_software_backend(self) -> omni_schema.SoftwareBackendEnum:
        """Get software backend of the benchmark"""

        return self.model.software_backend.code

    def get_software_environments(
        self,
    ) -> Dict[omni_schema.SoftwareEnvironmentId, omni_schema.SoftwareEnvironment]:
        """Get software environments"""

        environments = {}
        for env in self.model.software_environments:
            environments[env.id] = env

        return environments

    def get_definition(self) -> omni_schema.Benchmark:
        """Get underlying benchmark"""

        return self.model

    def get_easyconfigs(self):
        """Get easyconfigs"""
        easyconfigs = []
        for se in self.model.software_environments:
            easyconfigs.append(se["easyconfig"])
        return easyconfigs

    def get_conda_envs(self):
        """Get conda envs"""
        envs = []
        for se in self.model.software_environments:
            envs.append(se["conda"])
        return envs

    def get_stages(self) -> Dict[str, omni_schema.Stage]:
        """Get benchmark stages"""

        return dict([(x.id, x) for x in self.model.stages])

    def get_stage(self, stage_id: str) -> Optional[omni_schema.Stage]:
        """Get stage by stage_id"""

        return self.get_stages()[stage_id]

    def get_stage_by_output(self, output_id: str) -> Optional[omni_schema.Stage]:
        """Get stage that returns output with output_id"""

        stage_by_output: dict = {}
        for stage_id, stage in self.get_stages().items():
            stage_by_output.update({output.id: stage for output in stage.outputs})

        return stage_by_output.get(output_id)

    def get_modules_by_stage(
        self, stage: Union[str, omni_schema.Stage]
    ) -> Dict[str, omni_schema.Module]:
        """Get modules by stage/stage_id"""

        if isinstance(stage, str):
            stage = self.get_stages()[stage]

        return dict([(x.id, x) for x in stage.modules])

    def get_stage_implicit_inputs(
        self, stage: Union[str, omni_schema.Stage]
    ) -> List[str]:
        """Get implicit inputs of a stage by stage/stage_id"""

        if isinstance(stage, str):
            stage = self.get_stages()[stage]

        return [input.entries for input in stage.inputs]

    def get_explicit_inputs(self, input_ids: List[str]) -> Dict[str, str]:
        """Get explicit inputs of a stage by input_id(s)"""
        all_stages_outputs = []
        for stage_id in self.get_stages():
            outputs = self.get_stage_outputs(stage=stage_id)
            outputs = {
                key: value.format(
                    input="{input}",
                    stage=stage_id,
                    module="{module}",
                    params="{params}",
                    dataset="{dataset}",
                )
                for key, value in outputs.items()
            }
            all_stages_outputs.append(outputs)

        all_stages_outputs = merge_dict_list(all_stages_outputs)

        explicit = {key: None for key in input_ids}
        for in_deliverable in input_ids:
            # beware stage needs to be substituted
            curr_output = all_stages_outputs[in_deliverable]

            explicit[in_deliverable] = curr_output

        return explicit

    def get_stage_outputs(self, stage: Union[str, omni_schema.Stage]) -> Dict[str, str]:
        """Get outputs of a stage by stage/stage_id"""

        if isinstance(stage, str):
            stage = self.get_stages()[stage]

        return dict([(output.id, output.path) for output in stage.outputs])

    def get_output_stage(self, output_id: str) -> omni_schema.Stage:
        """Get stage that returns output with out_id"""

        stage_by_output: dict = {}
        for stage in self.model.stages:
            stage_by_output.update({out.id: stage for out in stage.outputs})

        return stage_by_output.get(output_id)

    def get_module_excludes(self, module: Union[str, omni_schema.Module]) -> List[str]:
        """Get module excludes by module/module_id"""

        if isinstance(module, str):
            module = self.get_modules()[module]

        return module.exclude

    def get_module_parameters(
        self, module: Union[str, omni_schema.Module]
    ) -> List[params.Params]:
        """Get module parameters by module/module_id"""

        if isinstance(module, str):
            module = self.get_modules()[module]

        parameters = None
        if module.parameters is not None:
            parameters = [
                params.Params.from_cli_args(x.values) for x in module.parameters
            ]

        return parameters

    def get_module_repository(
        self, module: Union[str, omni_schema.Module]
    ) -> omni_schema.Repository:
        """Get module repository by module/module_id"""

        if isinstance(module, str):
            module = self.get_modules()[module]

        return module.repository

    def get_module_environment(
        self, module: Union[str, omni_schema.Module]
    ) -> omni_schema.SoftwareEnvironmentId:
        """Get module software environment by module/module_id"""

        if isinstance(module, str):
            module = self.get_modules()[module]

        return module.software_environment

    def get_metric_collectors(self) -> List[omni_schema.MetricCollector]:
        return self.model.metric_collectors

    def is_initial(self, stage: omni_schema.Stage) -> bool:
        """Check if stage is initial"""

        if stage.inputs is None or len(stage.inputs) == 0:
            return True
        else:
            return False

    def get_outputs(self) -> Dict[str, str]:
        """Get outputs"""

        outputs = {}
        for stage_id, stage in self.get_stages().items():
            for output in stage.outputs:
                outputs[output.id] = output

        return outputs

    def get_modules(self) -> Dict[str, omni_schema.Module]:
        """Get modules"""

        modules = {}

        for stage_id, stage in self.get_stages().items():
            modules_in_stage = self.get_modules_by_stage(stage)
            modules.update(modules_in_stage)

        return modules



import logging
import os
import tarfile

from pathlib import Path
from filelock import FileLock
from git.exc import InvalidGitRepositoryError
from git import Repo
import requests

from omnibenchmark.workflow.snakemake.scripts.utils import (
    generate_unique_repo_folder_name,
)


# Get the code from a GitHub repository (https://docs.github.com/en/rest/repos/contents?apiVersion=2022-11-28#download-a-repository-archive-tar)
def get_git_archive_from_github(
    # TODO: deprecate, just use clone_git_repo
    output_dir: Path,
    repository_url: str,
    commit_hash: str,
) -> Path:
    # construct url
    baseurl = (
        repository_url.replace("https://", "")
        .replace("http://", "")
        .replace("github.com/", "")
        .replace(".git", "")
    )
    url = f"https://api.github.com/repos/{baseurl}/tarball/{commit_hash}"
    headers = {
        "Accept": "application/vnd.github+json",
        "X-GitHub-Api-Version": "2022-11-28",
    }
    # get the tar file
    req = requests.get(url, headers=headers)
    tarfilename = Path(f"{output_dir}.tar")
    with open(tarfilename, "wb") as f:
        f.write(req.content)
    # extract the tar file
    with tarfile.open(tarfilename) as f:
        members = f.getmembers()
        members_modified = []
        for member in members:
            if member.isfile():
                member.path = str(Path("/".join(Path(member.path).parts[1:])))
                members_modified.append(member)
        f.extractall(members=members_modified, path=output_dir, filter="data")
    if tarfilename.exists():
        os.remove(tarfilename)


# https://docs.gitlab.com/ee/api/repositories.html#get-file-archive
# TODO: deprecate, just use clone_git_repo
def get_git_archive_from_gitlab(
    output_dir: Path, repository_url: str, commit_hash: str
) -> Path:
    # construct url
    baseurl = repository_url.replace("https://", "").replace("http://", "")
    gitlab_remote = str(Path(baseurl).parents[-2])
    baseurl = baseurl.replace(f"{gitlab_remote}/", "")
    # url encode the baseurl
    baseurl = requests.utils.quote(baseurl, safe="")
    url = f"https://{gitlab_remote}/api/v4/projects/{str(baseurl)}/repository/archive?sha={commit_hash}"
    # get the tar file
    req = requests.get(url)
    tarfilename = Path(f"{output_dir}.tar")
    with open(tarfilename, "wb") as f:
        f.write(req.content)
    # extract the tar file
    with tarfile.open(tarfilename) as f:
        members = f.getmembers()
        members_modified = []
        for member in members:
            if member.isfile():
                member.path = str(Path("/".join(Path(member.path).parts[1:])))
                members_modified.append(member)
        f.extractall(members=members_modified, path=output_dir, filter="data")
    if tarfilename.exists():
        os.remove(tarfilename)


def get_git_archive(output_dir: Path, repository_url: str, commit_hash: str) -> Path:
    if "github.com" in repository_url:
        return get_git_archive_from_github(output_dir, repository_url, commit_hash)
    elif "gitlab" in repository_url:
        return get_git_archive_from_gitlab(output_dir, repository_url, commit_hash)
    try:
        # Fallback to direct git clone for unknown providers
        logging.info(
            f"Unknown git provider, falling back to direct clone: {repository_url}:{commit_hash}"
        )
        return clone_git_repo(output_dir, repository_url, commit_hash)

    except Exception as e:
        logging.error(f"Failed to clone repository: {e}")


def clone_git_repo(output_dir: Path, repository_url: str, commit_hash: str) -> Path:
    """
    Perform a shallow clone of a git repository at a specific commit.

    Args:
        output_dir: Directory where to clone the repository
        repository_url: URL of the git repository
        commit_hash: Specific commit hash to checkout
    """
    repo = Repo.clone_from(repository_url, output_dir.as_posix(), no_checkout=True)
    repo.git.checkout(commit_hash)
    return output_dir


# TODO: deprecate, just use clone_git_repo
def clone_module(output_dir: Path, repository_url: str, commit_hash: str) -> Path:
    module_name = generate_unique_repo_folder_name(repository_url, commit_hash)
    module_dir = output_dir / module_name

    lock = module_dir.with_suffix(".lock")
    with FileLock(lock):
        if not module_dir.exists():
            try:
                logging.info(
                    f"Get git archive `{repository_url}:{commit_hash}` to `{module_dir.as_posix()}`"
                )
                get_git_archive(module_dir, repository_url, commit_hash)
                observed_commit_hash = commit_hash
            except Exception:
                logging.info(
                    f"Archival retrieval failed, cloning module `{repository_url}:{commit_hash}` to `{module_dir.as_posix()}`"
                )
                repo = Repo.clone_from(repository_url, module_dir.as_posix())
                repo.git.checkout(commit_hash)
                observed_commit_hash = repo.head.commit.hexsha[:7]
        else:
            try:
                repo = Repo(module_dir.as_posix())
                observed_commit_hash = repo.head.commit.hexsha[:7]
            except InvalidGitRepositoryError:
                # is archive
                observed_commit_hash = commit_hash
            except Exception:
                observed_commit_hash = "known"

        if observed_commit_hash != commit_hash:
            logging.error(
                f"ERROR: Failed while cloning module `{repository_url}:{commit_hash}`"
            )
            logging.error(
                f"{commit_hash} does not match {repo.head.commit.hexsha[:7]}`"
            )
            raise RuntimeError(
                f"ERROR: {commit_hash} does not match {repo.head.commit.hexsha[:7]}"
            )

    return module_dir



"""
Common omniblock software management logics, regardless of conda/easybuild/system.
Calls workflow capabilities to reuse schema / yaml parsing logics
"""

import subprocess


def check_call(command: str, use_shell: bool = True) -> subprocess.CompletedProcess:
    return subprocess.run(
        command,
        text=True,
        capture_output=True,
        check=False,
        shell=use_shell,
    )


def check_lmod_status() -> subprocess.CompletedProcess:
    ret = subprocess.run(
        ["type", "module"], text=True, capture_output=True, check=False, shell=True
    )
    return ret


def check_singularity_status() -> subprocess.CompletedProcess:
    return check_call("singularity --version")


def check_easybuild_status() -> subprocess.CompletedProcess:
    return check_call("eb --version")


def check_conda_status() -> subprocess.CompletedProcess:
    return check_call("conda --version")



"""
Gathers performance benchmark.txt files and documents the parameters used for each stage/method, if applicable
"""

import csv
import glob
import os.path
from pathlib import Path
from typing import List

import pandas
import os.path as op


def write_combined_performance_file(out_dir: Path, performance_files: List[str]):
    if not os.path.exists(out_dir):
        os.mkdir(out_dir)

    fd = combine_performances(out_dir, performance_files)
    fd.to_csv(out_dir / "performances.tsv", sep="\t", index=False)


def combine_performances(
    out_dir: Path, performance_files: List[str]
) -> pandas.DataFrame:
    fd = pandas.DataFrame()

    for perf in performance_files:
        if os.path.exists(perf):
            curr = read_performance(perf)
            temp_df = pandas.DataFrame(curr, index=[1])
            temp_df["module"] = op.dirname(perf).split("/")[-2]
            temp_df["path"] = perf
            temp_df["params"] = read_params(out_dir, perf)
            fd = pandas.concat([fd, temp_df], ignore_index=True, axis=0)

    return fd


def read_performance(file_path: str):
    with open(file_path) as fh:
        reader = csv.DictReader(fh, delimiter="\t")
        for record in reader:
            record.pop("h:m:s", None)
            for k, v in record.items():
                # Check for NA or invalid values and handle them
                if v in ["NA", "none", "", "null"]:
                    record[k] = 0
                else:
                    try:
                        record[k] = float(v)
                    except ValueError:
                        record[k] = 0
            yield record


def tokenize(output_path: Path, file_path: str):
    ## we get only after the 'out' directory
    # TODO(ben): be more careful here
    try:
        fp = file_path.split(f"{output_path}/")[1].split("/")
    except IndexError:
        return []
    ## and slice in stage/method/params triples
    return [x for x in zip(*(iter(fp),) * 3)]


def read_params(output_path: Path, file_path: str):
    triples = tokenize(output_path, file_path)
    res = ""
    parent = output_path
    for triple in triples:
        parent = parent / triple[0] / triple[1] / triple[2]
        if "default" not in triple[2]:
            param_file_path = parent / "parameters.json"
            with open(param_file_path) as fh:
                params = fh.read()
                res = "%s %s %s %s %s;" % (
                    res,
                    triple[0],
                    triple[1],
                    triple[2],
                    params.strip(),
                )

    return res


if __name__ == "__main__":
    files = glob.glob("**/*_performance.txt")
    write_combined_performance_file(Path("out"), files)



#!/usr/bin/env python

"""
Easybuild-powered software management, mostly by omniblock. Also includes singularity image generation.

Izaskun Mallona
Started 5th June 2024
"""

import subprocess
import os
import os.path as op

from easybuild.tools.module_naming_scheme.utilities import det_full_ec_version
from easybuild.framework.easyconfig.tools import det_easyconfig_paths, parse_easyconfigs
from easybuild.tools.options import set_up_configuration


from importlib import resources as impresources


from omnibenchmark.config import config
from . import templates

HOME = op.expanduser("~")
## careful, 'all' will be prepended unless exported, shell-wise, the MODULEPATH/all path as MODULEPATH
## when running omb

MODULEPATH = config.get("easybuild", "MODULEPATH", default="")
ROBOTPATH = config.get("easybuild", "ROBOTPATH", default="")

if MODULEPATH != "":
    os.makedirs(MODULEPATH, exist_ok=True)

## shell-based stuff, partly to be replaced by direct eb API calls -------------------------------------


def generate_default_easybuild_config_arguments(
    modulepath: str = MODULEPATH,
    sourcepath: str = ROBOTPATH + ":" + os.getcwd(),
) -> str:
    args = "--installpath-modules=" + modulepath + " --sourcepath=" + sourcepath

    return args


set_up_configuration(
    args=["--info"] + generate_default_easybuild_config_arguments().split(" "),
    silent=True,
)


def construct_easybuild_easyconfig_command(easyconfig: str, threads: int = 2) -> str:
    """
    Constructs an easybuild command to build an easyconfig

     Args:
     - easyconfig (str): the easyconfig name. Doesn't have to be a full path. But readable from the robots path.
     - threads (int): number of threads to build the software
    """

    args = generate_default_easybuild_config_arguments()
    cmd = (
        """eb %(easyconfig)s --robot --parallel=%(threads)s %(args)s --detect-loaded-modules=unload --check-ebroot-env-vars=unset"""
        % {"easyconfig": op.basename(easyconfig), "threads": threads, "args": args}
    )

    return cmd


def easybuild_easyconfig(
    easyconfig: str,
    threads: int = 2,
) -> subprocess.CompletedProcess:
    """
    Easybuilds an easyconfig

     Args:
     - easyconfig (str): the easyconfig name. Doesn't have to be a full path. But readable from the robots path.
     - threads (int): number of threads to build the software
    """

    cmd = construct_easybuild_easyconfig_command(
        easyconfig=op.basename(easyconfig), threads=threads
    )

    ret = subprocess.run(
        cmd.split(" "), shell=False, capture_output=True, check=False, text=True
    )
    return ret


def parse_easyconfig(ec_fn: str) -> list:
    """
    Find and parse an easyconfig with specified filename,
    and return parsed easyconfig file (an EasyConfig instance).

    Args:
    - ec_fn (str): easyconfig filename. Doesn't have to be a full path. But readable from the robots path

    """
    # opts, _ = set_up_configuration(args = generate_default_easybuild_config_arguments(workdir).split(),
    #                                silent = True)

    ec_path = det_easyconfig_paths([ec_fn])[0]

    # parse easyconfig file;
    # the 'parse_easyconfigs' function expects a list of tuples,
    # where the second item indicates whether or not the easyconfig file was automatically generated or not
    ec_dicts, _ = parse_easyconfigs([(ec_path, False)])

    # only retain first parsed easyconfig
    return ec_path, ec_dicts[0]["ec"]


def get_easyconfig_full_path(easyconfig: str) -> str:
    """
    Finds the easyconfig full path
    Args:
    - easyconfig (str): easyconfig filename. Doesn't have to be a full path. But readable from the robots path
    """

    try:
        ec_path, ec = parse_easyconfig(op.basename(easyconfig))
        return ec_path
    except Exception:
        raise FileNotFoundError("\nERROR: easyconfig not found.\n")


def get_envmodule_name_from_easyconfig(easyconfig: str) -> str:
    """
    Returns the (standard) envmodulename from an easyconfig file.
    Args:
    - easyconfig (str): easyconfig filename. Doesn't have to be a full path. But readable from the robots path
    """
    ec_path, ec = parse_easyconfig(op.basename(easyconfig))
    return os.path.join(ec["name"], det_full_ec_version(ec))


def create_definition_file(
    easyconfig: str, singularity_recipe: str, envmodule: str, nthreads: int = 2
) -> None:
    """
    Materializes a Singularity recipe to build a given easyconfig, using a Singularity recipe template.
    Args:
    - easyconfig (str): easyconfig filename. Doesn't have to be a full path. But readable from the robots path
    - singularity_recipe (str): path to a Singularity template
    - envmodule (str): the envmodule name
    - nthreads (int): number of threads for easybuild
    """
    template = impresources.files(templates) / "ubuntu_jammy.txt"
    with open(template, "rt") as ubuntu, open(singularity_recipe, "w") as sing:
        for line in ubuntu.read().split("\n"):
            if "EASYCONFIG" in line:
                line = line.replace("EASYCONFIG", op.basename(easyconfig))
            if "ENVMODULENAME" in line:
                line = line.replace("ENVMODULENAME", envmodule)
            if "EASYBUILDNTHREADSINT" in line:
                line = line.replace("EASYBUILDNTHREADSINT", nthreads)
            sing.write(line + "\n")


def singularity_build(
    easyconfig: str, singularity_recipe: str
) -> subprocess.CompletedProcess:
    """
    Builds a Singularity recipe
    Args:
    - easyconfig (str): easyconfig filename. Doesn't have to be a full path. But readable from the robots path
    - singularity_recipe (str): path to a Singularity template
    """
    image_name = op.basename(easyconfig) + ".sif"

    cmd = (
        "singularity build --force --fakeroot " + image_name + " " + singularity_recipe
    )
    output = subprocess.run(
        cmd, shell=True, text=True, capture_output=True, check=False
    )
    return output


## untested,drafted 06 Aug 2024
def singularity_push(
    sif: str, docker_username: str, docker_password: str, oras_url: str
) -> subprocess.CompletedProcess | None:
    cmd = f"""singularity push --docker-username {docker_username} \
                 --docker-password {docker_password} \
                 {sif} \
                 {oras_url}"""
    try:
        subprocess.run(
            cmd.split(" "), shell=False, text=True, capture_output=True, check=True
        )
    except Exception as exc:
        raise RuntimeError("ERROR singularity build failed:", exc)
